# Copyright (c) 2017 by Cees van Wieringen <ceesvw@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
import os

import polib

import sys

import re
from django.conf import settings
from django.utils.translation import get_language


def clean_po_entry(po_entry):
    if 'fuzzy' in po_entry.flags:
        po_entry.flags.remove('fuzzy')


po_libs = {}


def get_po_libs(locale):
    if locale in po_libs:
        return po_libs[locale]
    po_libs[locale] = []
    for _po_file in find_pos(locale=locale):
        po_libs[locale].append(polib.pofile(_po_file))
    return po_libs[locale]


def get_ordered_path_list(include_djangos):
    paths = []

    # settings
    for localepath in settings.LOCALE_PATHS:
        if os.path.isdir(localepath):
            paths.append(localepath)

    # project/locale
    parts = settings.SETTINGS_MODULE.split('.')
    project = __import__(parts[0], {}, {}, [])
    projectpath = os.path.join(os.path.dirname(project.__file__), 'locale')
    localepaths = [os.path.normpath(path) for path in settings.LOCALE_PATHS]
    if (projectpath and os.path.isdir(projectpath) and
        os.path.normpath(projectpath) not in localepaths):
        paths.append(os.path.join(os.path.dirname(project.__file__), 'locale'))

    # project/app/locale
    for appname in settings.INSTALLED_APPS:
        appname = str(appname)  # to avoid a fail in __import__ sentence
        p = appname.rfind('.')
        if p >= 0:
            app = getattr(__import__(appname[:p], {}, {}, [appname[p + 1:]]), appname[p + 1:])
        else:
            app = __import__(appname, {}, {}, [])

        apppath = os.path.join(os.path.dirname(app.__file__), 'locale')

        if os.path.isdir(apppath):
            paths.append(apppath)

    # django/locale
    if include_djangos:
        paths.append(os.path.join(os.path.dirname(sys.modules[settings.__module__].__file__), 'locale'))

    return paths


def find_pos(locale, include_djangos=False):
    """
    scans a couple possible repositories of gettext catalogs for the given
    language code

    """
    paths = get_ordered_path_list(include_djangos)

    ret = []
    rx = re.compile(r'(\w+)/../\1')
    langs = (locale, )
    if u'-' in locale:
        _l, _c = map(lambda x: x.lower(), locale.split(u'-'))
        langs += (u'%s_%s' % (_l, _c), u'%s_%s' % (_l, _c.upper()), )
    elif u'_' in locale:
        _l, _c = map(lambda x: x.lower(), locale.split(u'_'))
        langs += (u'%s-%s' % (_l, _c), u'%s-%s' % (_l, _c.upper()), )

    for path in paths:
        for lang_ in langs:
            dirname = rx.sub(r'\1', '%s/%s/LC_MESSAGES/' % (path, lang_))
            for fn in ('django.po', 'djangojs.po', ):
                if os.path.isfile(dirname + fn) and os.path.abspath((dirname + fn)) not in ret:
                    ret.append(os.path.abspath(dirname + fn))
    return ret


def get_po_entry(msg_id, po_file=None):
    locale = get_language()
    if not po_file:
        for po_lib in get_po_libs(locale):
            po_entry = po_lib.find(msg_id)
            if po_entry:
                return po_entry, po_lib
    else:
        po_lib = polib.pofile(po_file)
        return po_lib.find(msg_id), po_lib
    return None, None


def update_po_file(po_lib, t9n_list=None):
    if t9n_list:
        for t9n in t9n_list:
            msgid = t9n.msgid
            msgctxt = t9n.msgctxt
            msgid_plural = t9n.plural

            po_entry = None
            if msgid:
                if msgctxt:
                    po_entry = po_lib.find(msgid, msgctxt=msgctxt)
                if not msgctxt and not po_entry:
                    # search for poentry with only a msgid if no msgctxt is available
                    po_entry = po_lib.find(msgid)

                if not po_entry:
                    # no po entry found create a new one.
                    po_entry = polib.POEntry(msgid=msgid, msgctxt=msgctxt, msgid_plural=msgid_plural)
                    po_lib.append(po_entry)

        po_lib.save()


def update_messages(locale, t9n_list=None):
    libs = get_po_libs(locale)
    if libs:
        update_po_file(libs[0], t9n_list=t9n_list)  # update first found po lib


import pytest

from django_l10n_extensions.management.commands import makemessages
from django_l10n_extensions.models.fields import T9N
from tests.conftest import _reload
from tests.testapp.models import TransTestModel


@pytest.mark.django_db
def test_collect_model_messages():
    collected_messages = {}

    def mock_update_messages(locale, t9n_list=None):
        print("locale: {}".format(locale))
        print("t9n_list: {}".format(t9n_list))
        messages = collected_messages.get(locale, [])
        messages += t9n_list
        collected_messages[locale] = messages

    # mock the update_messages with this test version to see which messages are all collected
    makemessages.update_messages = mock_update_messages
    TransTestModel.objects.create(trans_field=('mechanical device', 'spring'), other_trans_field='hurry')
    TransTestModel.objects.create(trans_field=('one', ))
    _reload(TransTestModel.objects.create(trans_field=('one', )))

    assert collected_messages == {}, "nothing collected yet, should still be empty."
    makemessages.Command().collect_model_messages({'locale': ['nl']})
    assert len(collected_messages.keys()) == 1 and 'nl' in collected_messages.keys(), \
        "Only 'nl' should be present in the collected messages keys"
    for t9n in [T9N('spring', msgctxt='mechanical device'), T9N('hurry'), T9N('one'), T9N('one')]:
        assert t9n in collected_messages['nl'], "{} not found in collected messages." \
                                                " Are msgid({}), msgctxt({}) and plural({}) the same?".\
            format(t9n,
                   "'{}'".format(t9n.msgid) if t9n.msgid else '',
                   "'{}'".format(t9n.msgctxt) if t9n.msgctxt else '',
                   "'{}'".format(t9n.plural) if t9n.plural else '',
                   )


# -*- coding: UTF-8 -*-
import logging
from argparse import RawTextHelpFormatter

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.management.commands import makemessages
from django.db.models import Model

from django_l10n_extensions.models.fields import TransField
from django_l10n_extensions.po_utils import update_messages

User = get_user_model()
log = logging.getLogger(__name__)


class Command(makemessages.Command):
    """
    Overwrite existing django makemigrations management command cause we want to force the LANGUAGE_CODE to 'en'
    """
    help = ("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
            "!! Overwrites existing django makemessages management command \n"
            "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
            "!!\n"
            "!! It adds collection messages from the TransField field of the models\n"
            "!!\n"
            "\n"
            "" + makemessages.Command.help)

    def create_parser(self, *args, **kwargs):
        parser = super(Command, self).create_parser(*args, **kwargs)
        parser.formatter_class = RawTextHelpFormatter
        return parser

    def add_arguments(self, parser):
        parser.add_argument('--no-models', dest='no-models', action='store_true',
            help="Don't collect the TransField translation, that is makemessages will run as the original Django's makemessages command")
        parser.add_argument('--models-only', dest='models-only', action='store_true',
            help="Only collect the model messages, that is skips the default behaviour of Django's makemessages command.")
        super(Command, self).add_arguments(parser)

    def handle(self, *app_labels, **options):
        if not options['no-models']:
            self.collect_model_messages(options)
        return super(Command, self).handle(*app_labels, **options)

    def collect_model_messages(self, options):
        # for the dutch language also update the translations, cause the database is leading for dutch translations
        locales = options['locale']
        if not locales:
            locales = [l[0] for l in settings.LANGUAGES]
        self.stdout.write("gathering django-l10n model translations for locales {}".format(locales))

        for subcls in Model.__subclasses__():
            if subcls.__module__ != '__fake__':
                trans_fields = [f for f in subcls._meta.fields if type(f) == TransField]

                if trans_fields:
                    log.debug('gathering translations for model {}'.format(subcls))
                    for trans_field in trans_fields:
                        t9n_list = [msg for msg in subcls.objects.values_list(trans_field.name, flat=True) if msg]
                        for locale in locales:
                            update_messages(locale, t9n_list)





# -*- coding: utf-8 -*-
import os

SECRET_KEY = 'dummy'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'django.contrib.sessions',
    'django_l10n_extensions',
    'tests.testapp',
]

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

PROJECT_DIR = os.path.abspath(os.path.dirname(__file__))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_DIR, 'l10n_extensions.db'),
        # 'NAME': ':memory:',
    }
}

MEDIA_ROOT = '/tmp/django_l10n_extensions_test_media/'

MEDIA_PATH = '/media/'

ROOT_URLCONF = 'tests.testapp.urls'

DEBUG = True

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'debug': True,
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

TESTAPP_DIR = os.path.dirname(__file__)

LOCALE_PATHS = [
    os.path.join(TESTAPP_DIR, 'locale'),
]
L10N = True
LANGUAGES = (
    ('nl', u'Nederlands'),
    ('fr', u'French'),
    ('en', u'English'),
)


# coding=utf-8
import pytest

from django_l10n_extensions.measures import Distance, Area, Volume, Weight, Temperature


def test_compare_area_operations():
    area_1 = Area(ha=11.000)
    area_2 = Area(ha=11)
    assert area_1 == area_2

    assert (area_1 + area_2).ha == 22.0000
    assert (area_1 - area_2).ha == 0.0000
    assert (area_1 / 11).ha == 1.0000
    assert (area_1 / 11).sq_m == 10000.0000
    assert (area_1 * 11).ha == 121.0000
    assert (area_1 * 11).sq_m == 1210000.0000

    assert area_1 > Area(ha=2)
    assert area_1 < Area(ha=22)

    area_sum = sum([Area(ha=3), Area(5), Area(2)])
    assert area_sum.ha == 3.0007

    assert abs(round(Area(1).l10n_value / 3, 2) - 0.33) <  0.0001


def test_constructors():
    assert Distance(3).m == 3
    assert Distance(3).dm == 30
    assert Distance(m=4).m == 4

    assert Distance({'unit':'m', 'value':3}).m, 3
    with pytest.raises(ValueError):
        assert Distance({'distance':'m', 'value':3}).m == 3


def test_weight():
    assert Weight(1).g == 1
    assert Weight(kg=1).g == 1000
    assert abs(Weight(kg=4.5).us_ton - 0.0049604009) < 0.00001
    assert abs(Weight(kg=4.5).short_ton - 0.0049604009) < 0.00001
    assert Weight(lb=4.5).pound == 4.5


def test_volume():
    assert Volume(1).l == 1
    assert abs(Volume(l=1).cu_dm - 1) < 0.000001
    assert Volume(gal=1).l == 3.78541178
    assert abs(Volume(ml=44).gal - 0.0116235703) < 0.0001
    assert Volume(cu_m=5).l == 5000
    assert abs(Volume(cu_yd=3).cu_yd - 3) < 0.000001


def test_temperature():
    assert Temperature(C=5).C == 5
    assert Temperature(C=5).K == 278.15
    assert Temperature(C=5).F == 41

    assert Temperature(F=5).C == -15
    assert Temperature(F=5).K == 258.15
    assert Temperature(F=5).F == 5


def test_add():
    assert Volume(l=1) + Volume(ml=100) == Volume(l=1.1)
    with pytest.raises(TypeError):
        assert Volume(l=1) + Weight(kg=1), 'can not add up Volumes and Weight'
    assert Distance(5) + Distance(hm=5) == Distance(m=505)
    with pytest.raises(TypeError):
        assert Distance(km=3) + 4, 'can not add plain numeric values (in what unit is the numeric value?'

def test_mul():
    assert Distance(5) * 2 == Distance(10)
    assert Distance(m=5) * Distance(km=0.01) == Area(sq_m=50)
    assert Area(sq_m=2) * 4 == Area(sq_m=8)
    assert Area(sq_m=2) * Distance(km=0.02) == Volume(cu_m=40)
    assert Distance(m=15) * Area(sq_m=3) == Volume(cu_m=45)
    assert Area(ha=1) * 3 == Area(ha=3)
    assert 3 * Area(ha=1) == Area(ha=3)
    with pytest.raises(TypeError):
        assert Area(ha=1) * Area(ha=1)


from django.utils.timezone import now
from django.views.generic import TemplateView


class TransExampleView(TemplateView):
    def get_context_data(self, **kwargs):
        context_data = super(TransExampleView, self).get_context_data(**kwargs)
        context_data['date'] = now()
        context_data['numbers'] = [0,1,2,3,5,10,20]
        return context_data


# coding=utf-8
from copy import deepcopy

from django.contrib.gis.measure import MeasureBase, NUMERIC_TYPES, Distance as GisDistance, Area as GisArea, \
    AREA_PREFIX, pretty_name
from django.utils.formats import number_format

from django_l10n_extensions.l10n_threading import get_l10n

VOLUME_PREFIX = 'cu_'


class MeasureL10nBase(MeasureBase):

    UNITS_REPR = {}

    def __init__(self, value=None, default_unit=None, decimal_pos=2, **kwargs):
        if value is not None:
            if isinstance(value, NUMERIC_TYPES + (str,)):
                unit = self.get_unit() or default_unit or self.STANDARD_UNIT
                kwargs[unit] = value
            else:
                # assume value is in the form of a dict like {'value': 1.0, 'unit': 'ha'}
                try:
                    kwargs[value['unit']] = value['value']
                except KeyError:
                    raise ValueError(u'invalid constructor parameter {}'.format(value))
            self._default_unit = default_unit
        self.decimal_pos = decimal_pos
        super(MeasureL10nBase, self).__init__(default_unit, **kwargs)

    @property
    def default_value(self):
        return getattr(self, self._default_unit)

    @property
    def l10n_value(self):
        return self.get_l10n_value()

    def get_l10n_value(self, unit=None):
        if not unit:
            unit = self.get_unit()
        return self.__getattr__(unit)

    @property
    def l10n_unit(self):
        unit = self.get_unit()
        return self.UNITS_REPR.get(unit, unit)

    @property
    def l10n(self):
        return self.as_l10n()

    def as_dict(self):
        return {u"value": self.l10n_value, u"unit": self.l10n_unit}

    def as_l10n(self, decimal_pos=None):
        value = number_format(self.l10n_value, decimal_pos=decimal_pos if decimal_pos is not None else self.decimal_pos)
        return u'{0} {1}'.format(value, self.l10n_unit)

    def get_unit(self):
        return self.DEFAULT_UNIT

    def __radd__(self, other):
        if other == 0:  # a little tricky cause the sum() operations starts with the value 0!
            other = Area(0)
        return self + other

    def __deepcopy__(self, memo):
        class_instance = self.__class__
        copied_class = class_instance.__new__(class_instance)
        memo[id(self)] = copied_class
        for field_name, field_instance in self.__dict__.items():
            setattr(copied_class, field_name, deepcopy(field_instance, memo))
        return copied_class

    def __getattr__(self, name):
        try:
            return super(MeasureL10nBase, self).__getattr__(self.unit_attname(name))
        except Exception as e:
            raise AttributeError(e)


class Distance(MeasureL10nBase, GisDistance):
    METER = u'm'
    FOOT = u'ft'
    YARD = u'yard'
    DEFAULT_UNIT = METER

    UNITS = GisDistance.UNITS.copy()
    UNITS.update({
        'hm': 100,
        'dm': 0.1,
    })

    def __init__(self, value=None, default_unit=DEFAULT_UNIT, **kwargs):
        super(Distance, self).__init__(value, default_unit, **kwargs)

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_distance if l10n else self._default_unit
        return unit

    def __mul__(self, other):
        if isinstance(other, Area):
            return Volume(default_unit=VOLUME_PREFIX + self._default_unit,
                **{VOLUME_PREFIX + self.STANDARD_UNIT: (self.standard * other.standard)})
        else:
            return super(Distance, self).__mul__(other)


class Area(MeasureL10nBase, GisArea):
    SQUARE_METER = AREA_PREFIX + Distance.METER
    SQUARE_FOOT = AREA_PREFIX + Distance.FOOT
    ACRE = u'acre'
    FEDDAN = u'feddan'
    HECTARE = u'ha'
    MU = u'mu'
    DEFAULT_UNIT = AREA_PREFIX + Distance.STANDARD_UNIT

    UNITS = GisArea.UNITS.copy()
    UNITS.update({
        HECTARE: 10000.0,
        ACRE: 10000 / 2.471,
        FEDDAN: 10000 / 2.381,
        MU: 10000 / 15.0,
    })
    ALIAS = GisArea.ALIAS.copy()
    ALIAS.update({
        u'hectare': u'ha',
        u'acre': u'acre',
        u'feddan': u'feddan',
        u'mŭ': u'mu',
    })
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}
    UNITS_REPR = {
        MU: u'mŭ',
        HECTARE: u'ha',
        ACRE: u'acre',
        FEDDAN: u'feddan',
    }

    def __init__(self, value=None, default_unit=DEFAULT_UNIT, decimal_pos=1, **kwargs):
        super(Area, self).__init__(value, default_unit, decimal_pos=decimal_pos, **kwargs)

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_area if l10n else self._default_unit
        return unit

    def __mul__(self, other):
        if isinstance(other, Distance):
            return Volume(default_unit=VOLUME_PREFIX + self._default_unit[len(AREA_PREFIX):],
                **{VOLUME_PREFIX + self.STANDARD_UNIT[len(AREA_PREFIX): ]: (self.standard * other.standard)})
        elif isinstance(other, NUMERIC_TYPES):
            return self.__class__(default_unit=self._default_unit,
                                  **{self.STANDARD_UNIT: (self.standard * other)})
        else:
            raise TypeError('{area} must be multiplied with number or {distance}' % {
                'area': pretty_name(self.__class__),
                'distance': pretty_name(Distance),
            })



# weight
# 1000 g = 0,00220462 lb = 2 kati
class Weight(MeasureL10nBase):
    GRAM = u'g'
    KG = u'kg'
    MG = u'mg'
    TON = u'ton'
    POUND = u'lb'
    SHORT_TON = u'short ton'
    MARKET_CATTY = u'市斤'

    DEFAULT_UNIT = GRAM
    STANDARD_UNIT = GRAM  # this is the base unit and its value is used to recalculate other unit values
    UNITS = {
        GRAM: 1,
        KG: 1000,
        MG: 0.001,
        TON: 1000000,
        POUND: 453.59237,
        SHORT_TON: 907184.75,
        MARKET_CATTY: 500,
    }
    ALIAS = {
        u'gram': GRAM,
        u'kilogram': KG,
        u'milligram': MG,
        u'tonne': TON,
        u'US_ton': SHORT_TON,
        u'short_ton': SHORT_TON,
        u'lbm': POUND,
        u'pound': POUND,
        u'kati': MARKET_CATTY,
    }
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}
    UNITS_REPR = {
        GRAM: u'g',
        KG: u'kg',
        MG: u'mg',
        TON: u'ton',
        POUND: u'lb',
        SHORT_TON: u'short ton',
        MARKET_CATTY: u'市斤',
    }

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_weight if l10n else self._default_unit
        return unit


# volume
# 1 l = 0,264172 gal = 2,11338 pt = 33,814 oz
class Volume(MeasureL10nBase):
    LITER = u'l'

    GALLON = u'gal'
    DL = u'dl'
    CL = u'cl'
    ML = u'ml'

    DEFAULT_UNIT = LITER
    STANDARD_UNIT = VOLUME_PREFIX + Distance.STANDARD_UNIT  # this is the base unit and its value is used to recalculate other unit values
    UNITS = {'%s%s' % (VOLUME_PREFIX, k): v ** 3 for k, v in Distance.UNITS.items()}
    UNITS.update({
        LITER: 0.001,
        GALLON: 0.00378541178,
        CL: 0.00001,
        ML: 0.000001,
    })
    ALIAS = {
        u'liter': LITER,
        u'gallon': GALLON,
        u'deciliter': DL,
        u'centiliter': CL,
        u'milliliter': ML,
    }
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}
    UNITS_REPR = {
        LITER: u'l',
        GALLON: u'gal',
        ML: u'ml',
    }

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_volume if l10n else self._default_unit
        return unit


class Temperature(MeasureL10nBase):
    CELSIUS = u'C'
    FAHRENHEIT = u'F'
    KELVIN = u'K'

    DEFAULT_UNIT = CELSIUS
    STANDARD_UNIT = CELSIUS  # this is the base unit and its value is used to recalculate other unit values
    UNITS = {
        CELSIUS: lambda x: x,
        FAHRENHEIT: lambda x: x * 9.0 / 5.0 + 32.0,
        KELVIN: lambda x: x + 273.15
    }
    UNITS_REVERSE = {
        CELSIUS: lambda x: x,
        FAHRENHEIT: lambda x: (x - 32) / 9.0 * 5.0,
        KELVIN: lambda x: x - 273.15
    }
    UNITS_REPR = {
        CELSIUS: u'°C',
        FAHRENHEIT: u'°F',
        KELVIN: u'K',
    }
    ALIAS = {
        u'Celsius': CELSIUS,
        u'Fahrenheit': FAHRENHEIT,
        u'Kelvin': KELVIN,
    }
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}

    def __getattr__(self, name):
        try:
            return self.UNITS[self.unit_attname(name)](self.standard)
        except Exception as e:
            raise AttributeError(e)

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_temp if l10n else self._default_unit
        return unit

    def default_units(self, kwargs):
        """
        Return the unit value and the default units specified
        from the given keyword arguments dictionary.
        """
        val = 0.0
        default_unit = self.STANDARD_UNIT
        for unit, value in kwargs.items():
            if not isinstance(value, float):
                value = float(value)
            if unit in self.UNITS:
                val = self.UNITS_REVERSE[unit](value)
            elif unit in self.ALIAS:
                u = self.ALIAS[unit]
                val = self.UNITS_REVERSE[u](value)
            else:
                lower = unit.lower()
                if lower in self.UNITS:
                    val = self.UNITS_REVERSE[lower](value)
                elif lower in self.LALIAS:
                    u = self.LALIAS[lower]
                    val = self.UNITS_REVERSE[u](value)
                else:
                    raise AttributeError(u'Unknown unit type: %s' % unit)
        return val, default_unit

    def __radd__(self, other):
        pass


class Velocity(MeasureL10nBase):
    MPS = u'mps'
    KMH = u'kmh'
    MPH = u'mph'

    DEFAULT_UNIT = MPS
    STANDARD_UNIT = MPS  # this is the base unit and its value is used to recalculate other unit values
    UNITS = {
        MPS: 1,
        KMH: 1000.0/3600.0,
        MPH: 0.44704,  # from Wikipedia
    }
    ALIAS = {
        u'm/s': MPS,
        u'km/h': KMH,
        u'mph': MPH,
    }
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}
    UNITS_REPR = {
        MPS: u'm/s',
        KMH: u'km/h',
        MPH: u'mph',
    }

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_velocity if l10n else self._default_unit
        return unit


class Precipitation(MeasureL10nBase):
    MM = u'mm'
    INCH = u'inch'

    DEFAULT_UNIT = MM
    STANDARD_UNIT = MM  # this is the base unit and its value is used to recalculate other unit values
    UNITS = {
        MM: 1,
        INCH: 25.4,
    }
    ALIAS = {
        u'mm': MM,
        u'mm water': MM,
        u'inch': INCH,
        u'inches water': INCH,
    }
    LALIAS = {k.lower(): v for k, v in ALIAS.items()}
    UNITS_REPR = {
        MM: u'mm',
        INCH: u'inch',
    }

    def get_unit(self):
        l10n = get_l10n()
        unit = l10n.unit_precipitation if l10n else self._default_unit
        return unit




import json

from django.db import models
from django.utils.translation import gettext, pgettext, npgettext, ngettext

from django_l10n_extensions.forms import fields
from django_l10n_extensions.exceptions import L10NException
from django_l10n_extensions import measures


class T9N(object):
    def __init__(self, msgid=None, msgctxt=None, msgid_plural=None, **kwargs):
        self.msgid = msgid or kwargs.get('i')
        self.msgctxt = msgctxt or kwargs.get('c')
        self.plural = msgid_plural or kwargs.get('p')
        if not self.msgid:
            raise ValueError("Invalid arguments passed, need at least a msgid")

    def __str__(self):
        if self.msgctxt:
            return pgettext(self.msgctxt, self.msgid)
        return gettext(self.msgid)

    def trans(self, count):
        if self.msgctxt:
            return npgettext(self.msgctxt, self.msgid, self.plural, count)
        return ngettext(self.msgid, self.plural, count)

    def __eq__(self, other):
        if not other:
            return False
        if isinstance(other, self.__class__):
            return other.msgid == self.msgid and other.msgctxt == self.msgctxt and other.plural == self.plural
        return False

    def __unicode__(self):
        return self.__str__()

    def __repr__(self):
        return self.__str__()

    def __len__(self):
        return len(self.__str__())


class TransField(models.CharField):

    def from_db_value(self, value, expression, connection):
        return self.to_python(value)

    def to_python(self, value):
        if isinstance(value, T9N) or not value  :
            return value
        try:
            return T9N(**json.loads(value))
        except ValueError:
            # db value isn't in json format, assigned the complete string as msgid.
            return T9N(msgid=value)

    def get_prep_value(self, value):
        if not value:
            return super(TransField, self).get_prep_value(value)
        data = {}
        if isinstance(value, T9N):
            data = {'i': value.msgid, 'c': value.msgctxt, 'p': value.plural}
        elif isinstance(value, (tuple, list)):
            length = len(value)
            if length == 1:
                data['i'] = value[0]
            elif length == 2:
                data['c'] = value[0]
                data['i'] = value[1]
            elif length == 3:
                data['c'] = value[0]
                data['i'] = value[1]
                data['p'] = value[2]
        elif isinstance(value, dict):
            data['i'] = value.get('msgid') or value.get('i')
            data['c'] = value.get('msgctxt') or value.get('c')
            data['p'] = value.get('plural') or value.get('p')
        else:
            data['i'] = value
        return json.dumps(data)


class BaseMeasureField(models.FloatField):
    measure_class = None
    DEFAULT_UNIT = measures.MeasureBase.STANDARD_UNIT

    def construct_measure(self, value, unit=None):
        if not self.measure_class:
            raise L10NException('A measure class is required for {0}'.format(self.__class__))
        if not self.DEFAULT_UNIT:
            raise L10NException('A DEFAULT_UNIT is required for {0}'.format(self.__class__))
        return self.measure_class(**{unit if unit else self.DEFAULT_UNIT: value})

    def from_db_value(self, value, expression, connection):
        if type(value) in [float, int]:
            return self.construct_measure(value)

    def to_python(self, value):
        if isinstance(value, self.measure_class):
            return value
        if value is not None:
            return self.measure_class(value)

    def get_prep_value(self, value):
        if value is None:
            return None
        if isinstance(value, self.measure_class):
            value = getattr(value, self.DEFAULT_UNIT)
        else:
            value = float(value)
            unit = self.measure_class().get_unit()
            if unit != self.DEFAULT_UNIT:
                measure = self.construct_measure(value, unit=unit)
                value = measure.default_value
        return value


class BaseDecimalMeasureField(models.DecimalField):
    measure_class = None
    DEFAULT_UNIT = measures.MeasureBase.STANDARD_UNIT

    def construct_measure(self, value, unit=None):
        if not self.measure_class:
            raise L10NException('A measure class is required for {0}'.format(self.__class__))
        if not self.DEFAULT_UNIT:
            raise L10NException('A DEFAULT_UNIT is required for {0}'.format(self.__class__))
        return self.measure_class(**{unit if unit else self.DEFAULT_UNIT: value})

    def from_db_value(self, value, expression, connection):
        if type(value) in [float, int]:
            return self.construct_measure(value)

    def to_python(self, value):
        if isinstance(value, self.measure_class):
            return value
        if value is not None:
            return self.measure_class(value)

    def get_prep_value(self, value):
        if value is None:
            return None
        if isinstance(value, self.measure_class):
            value = getattr(value, self.DEFAULT_UNIT)
        else:
            value = float(value)
            unit = self.measure_class().get_unit()
            if unit != self.DEFAULT_UNIT:
                measure = self.construct_measure(value, unit=unit)
                value = measure.default_value
        return value


class DistanceField(BaseMeasureField):
    measure_class = measures.Distance
    DEFAULT_UNIT = measures.Distance.METER

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.DistanceFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class AreaField(BaseMeasureField):
    measure_class = measures.Area
    DEFAULT_UNIT = measures.Area.DEFAULT_UNIT

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.AreaFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class WeightField(BaseMeasureField):
    measure_class = measures.Weight
    DEFAULT_UNIT = measures.Weight.DEFAULT_UNIT

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.WeightFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class VolumeField(BaseMeasureField):
    measure_class = measures.Volume
    DEFAULT_UNIT = measures.Volume.DEFAULT_UNIT

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.VolumeFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class TemperatureField(BaseMeasureField):
    measure_class = measures.Temperature
    DEFAULT_UNIT = measures.Temperature.CELSIUS

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.TemperatureFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class VelocityField(BaseMeasureField):
    measure_class = measures.Velocity
    DEFAULT_UNIT = measures.Velocity.MPS

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.VelocityFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)


class PrecipitationField(BaseMeasureField):
    measure_class = measures.Precipitation
    DEFAULT_UNIT = measures.Precipitation.MM

    def formfield(self, **kwargs):
        defaults = {'form_class': fields.PrecipitationFormField}
        defaults.update(kwargs)
        return super(models.FloatField, self).formfield(**defaults)



# Copyright (c) 2017 by Cees van Wieringen <ceesvw@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
"""

You have to define an AUTO_RELOAD_METHOD in your settings or
DEFAULT_AUTO_RELOAD_METHOD will be used.

Options for this settings::
  * "test" for a django instance (this do a touch over settings.py for reload)
  * "apache2"
  * "httpd"
  * "wsgi"
  * "restart_script <script_path_name>"

"""
from django.core.exceptions import ImproperlyConfigured

DEFAULT_ENABLE_INLINE_TRANS = False
DEFAULT_AUTO_RELOAD_METHOD = 'test'
DEFAULT_AUTO_RELOAD_TIME = '5'
DEFAULT_AUTO_RELOAD_LOG = 'var/log/autoreload_last.log'


def use_inline_trans():
    from django.conf import settings
    try:
        return settings.ENABLE_INLINE_TRANS
    except (ImproperlyConfigured, AttributeError):
        return DEFAULT_ENABLE_INLINE_TRANS


def user_can_update(user):
    return use_inline_trans() and user.is_staff


def get_auto_reload_method():
    from django.conf import settings
    try:
        return settings.AUTO_RELOAD_METHOD
    except (ImproperlyConfigured, AttributeError):
        return DEFAULT_AUTO_RELOAD_METHOD


def get_auto_reload_time():
    from django.conf import settings
    try:
        return settings.AUTO_RELOAD_TIME
    except (ImproperlyConfigured, AttributeError):
        return DEFAULT_AUTO_RELOAD_TIME


def get_auto_reload_log():
    from django.conf import settings
    try:
        return settings.AUTO_RELOAD_LOG
    except (ImproperlyConfigured, AttributeError):
        return DEFAULT_AUTO_RELOAD_LOG



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models import CharField

from django_l10n_extensions.models import fields as l10n_fields


class TransTestModel(models.Model):
    trans_field = l10n_fields.TransField(max_length=128)
    other_trans_field = l10n_fields.TransField(max_length=128)


class MeasuresTestModel(models.Model):
    name = CharField(max_length=51)  # 'My units'
    area = l10n_fields.AreaField(null=True, blank=True)
    height = l10n_fields.DistanceField(null=True, blank=True)
    temp = l10n_fields.TemperatureField(null=True, blank=True)
    windspeed = l10n_fields.VelocityField(null=True, blank=True)
    precipitation = l10n_fields.PrecipitationField(null=True, blank=True)



# Copyright (c) 2017 by Cees van Wieringen <ceesvw@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.


from django.views.generic.base import View


from polib import POEntry

from django_l10n_extensions.mixins import JSONResponseMixin
from django_l10n_extensions.po_utils import get_po_entry
from django_l10n_extensions.settings import user_can_update


def po_entry_to_json(po_entry):
    if po_entry is None:
        return
    assert isinstance(po_entry, POEntry), "can only serialize POEntry instances, got a {} ({})".format(type(po_entry), po_entry)
    return {
            'msgid': po_entry.msgid,
            'msgctxt': po_entry.msgctxt if po_entry.msgctxt else '',
            'msgstr': po_entry.msgstr if po_entry.msgstr  else '',
            'flags': po_entry.flags,
            'translated': po_entry.translated()
        }


class PoEntryView(JSONResponseMixin, View):

    def get(self, request):
        msgid = request.GET.get('msgid')
        if msgid:
            po_entry, po_lib = get_po_entry(msgid)
            return self.render_to_response(context=po_entry_to_json(po_entry))


class PoEntryUpdateView(JSONResponseMixin, View):

    def post(self, request):
        result = {
            'updated': False
        }
        errors = []

        if not user_can_update(request.user):
            errors.append("you don't have sufficient rights to update translations.")
        else:
            msgid = request.POST.get('msgid')
            msgstr = request.POST.get('msgstr')
            if not msgid:
                errors.append("no msgid found can't update if no msgid provided")
            if not msgstr:
                errors.append("no translated msgstr found can't update with empty translation")
            if not errors:
                po_entry, po_lib = get_po_entry(msgid)
                if po_entry:
                    po_entry.msgstr = msgstr
                    result['updated'] = True
                    result['poentry'] = po_entry_to_json(po_entry)
                    po_lib.save()
                else:
                    errors.append("no translation message found for '{}', can't update unknown translation".format(msgid))
        result['errors'] = errors
        return self.render_to_response(context=result)


# def do_restart(request):
#     """
#     * "test" for a django instance (this do a touch over settings.py for reload)
#     * "apache"
#     * "httpd"
#     * "wsgi"
#     * "restart_script <script_path_name>"
#     """
#     if get_user_can_translate(request.user):
#         reload_method = get_auto_reload_method()
#         reload_log = get_auto_reload_log()
#         reload_time = get_auto_reload_time()
#         command = "echo no script"
#         if reload_method == 'test':
#             command = 'touch %s' % os.path.join(settings.BASEDIR, 'settings.py')
        ## No RedHAT or similars
        # elif reload_method == 'apache2':
        #     command = 'sudo apache2ctl restart'
        ## RedHAT, CentOS
        # elif reload_method == 'httpd':
        #     command = 'sudo service httpd restart'
        #
        # elif reload_method.startswith('restart_script'):
        #     command = ' '.join(reload_method.split(" ")[1:])
        # if os.path.exists(os.path.dirname(reload_log)):
        #     os.system("sleep 2 && %s &> %s & " % (command, reload_log))
        # else:
        #     print('The AUTO_RELOAD_LOG directory do not exist')  # Just in case our stdout is logged somewhere
        #     os.system("sleep 2 && %s & " % command)
        # return render_to_response('inlinetrans/response.html',
        #                           {'message': reload_time},
        #                           context_instance=RequestContext(request))

#    return HttpResponseRedirect(request.environ['HTTP_REFERER'])



# coding=utf-8
import pytest

from django_l10n_extensions.l10n_threading import activate as activate_l10n
from django_l10n_extensions.measures import Distance


@pytest.mark.django_db
def test_l10n(l10n_nl, l10n_us):
    activate_l10n(l10n_nl)
    distance = Distance(11)
    assert distance.m == 11
    assert abs(distance.yard - 12.02974) < 0.0001
    assert distance.l10n == '11.00 m'
    assert distance.as_l10n(decimal_pos=0) == '11 m'
    assert distance.l10n_value == 11
    assert distance.l10n_unit == 'm'

    # change l10n
    # TODO punctuation should alter
    activate_l10n(l10n_us)
    assert distance.l10n == '12.02 yard'
    assert abs(distance.l10n_value - 12.02974) < 0.0001
    assert distance.l10n_unit == 'yard'



from django.forms.fields import Field


class DistanceFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Distance
        return Distance(value)


class AreaFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Area
        return Area(value)


class WeightFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Weight
        return Weight(value)


class VolumeFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Volume
        return Volume(value)


class TemperatureFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Temperature
        return Temperature(value)


class VelocityFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Velocity
        return Velocity(value)

class PrecipitationFormField(Field):

    def to_python(self, value):
        from django_l10n_extensions.measures import Precipitation
        return Precipitation(value)



# Copyright (c) 2010-2013 by Yaco Sistemas <ant30tx@gmail.com> or <goinnn@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include, url

from django_l10n_extensions.views import PoEntryView, PoEntryUpdateView

urlpatterns = [
    # url(r'^apply_changes/$', 'do_restart', name='apply_changes'),
    url(r'^po_entry/$', PoEntryView.as_view(), name='get_po_entry'),
    url(r'^po_entry/update$', PoEntryUpdateView.as_view(), name='update_poentry_url'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-04-26 08:51
from __future__ import unicode_literals

from django.db import migrations, models
import django_l10n_extensions.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MeasuresTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=51)),
                ('area', django_l10n_extensions.models.fields.AreaField(blank=True, null=True)),
                ('height', django_l10n_extensions.models.fields.DistanceField(blank=True, null=True)),
                ('temp', django_l10n_extensions.models.fields.TemperatureField(blank=True, null=True)),
                ('windspeed', django_l10n_extensions.models.fields.VelocityField(blank=True, null=True)),
                ('precipitation', django_l10n_extensions.models.fields.PrecipitationField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='TransTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('trans_field', django_l10n_extensions.models.fields.TransField(max_length=128)),
                ('other_trans_field', django_l10n_extensions.models.fields.TransField(max_length=128)),
            ],
        ),
    ]



# Copyright (c) 2017 by Cees van Wieringen <ceesvw@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
import pkg_resources
from django.templatetags import i18n

from django_l10n_extensions.po_utils import get_po_entry
from django_l10n_extensions.settings import use_inline_trans
from .utils import html_escape


def get_version(svn=False):
    """Returns the version as a human-format string."""
    v = pkg_resources.require("django_l10n_extensions")[0].version
    if svn:
        from django.utils.version import get_svn_revision
        import os
        svn_rev = get_svn_revision(os.path.dirname(__file__))
        if svn_rev:
            v = '%s-%s' % (v, svn_rev)
    return v


def to_html(msgid, value, html_tag='span'):
    po_entry, po_lib = get_po_entry(msgid)

    return u"<{html_tag} data-msgid='{msgid}' class='_itr {not_translated}'>{value}</{html_tag}>".format(
        msgid=html_escape(msgid),
        value=value,
        not_translated='not_translated' if not po_entry or not po_entry.translated() else '',
        html_tag=html_tag
    )


class InlineTranslateNode(i18n.TranslateNode):

    def render(self, context):
        splitted = self.token.split_contents()
        tag_name, msgid = splitted[0], splitted[1]
        value = super(InlineTranslateNode, self).render(context)
        return to_html(msgid[1:-1], value)  # [1:-1] to strip of first and last "-char


class InlineBlockTranslateNode(i18n.BlockTranslateNode):

    def render(self, context, nested=False):
        singular, vars = self.render_token_list(self.singular)
        value = super(InlineBlockTranslateNode, self).render(context, nested)
        return to_html(singular, value, html_tag='div')


if use_inline_trans():
    i18n.TranslateNode = InlineTranslateNode
    i18n.BlockTranslateNode = InlineBlockTranslateNode



import threading
from copy import deepcopy


_local = threading.local()


def get_l10n():
    """
    :return: active l10n. First look in local thread if not present check user localization
    """
    local_l10n = getattr(_local, "l10n_instance", None)
    if local_l10n:
        return local_l10n

    from django_l10n_extensions.models.models import DEFAULT_L10N
    l10n = deepcopy(DEFAULT_L10N)
    return l10n


def activate(l10n):
    if not l10n:
        return
    _local.l10n_instance = l10n


def deactivate():
    if hasattr(_local, "l10n_instance"):
        del _local.l10n_instance



import json

import pytest
from django.utils.translation import activate, deactivate
from django.utils.translation import gettext as _

from django_l10n_extensions.models.fields import T9N, TransField
from tests.conftest import _reload
from tests.testapp.models import TransTestModel


def test_sanity_translaiton():
    activate('en')
    assert _('edit') == 'edit', "sanity test to test django translation (en) is working"
    deactivate()

    activate('nl')
    assert _('edit') == 'bewerken', "sanity test to test django translation (nl) is working"
    deactivate()


@pytest.mark.django_db
def test_trans_field():
    # we need to reload the instance from the db, so django converts the varchar value to a I18N object
    t9n_model = _reload(TransTestModel.objects.create(trans_field='edit'))
    activate('en')
    assert str(t9n_model.trans_field) == 'edit'
    assert "In a string '{}'".format(t9n_model.trans_field) == "In a string 'edit'"
    assert t9n_model.trans_field.msgid == 'edit'
    deactivate()

    activate('nl')
    assert str(t9n_model.trans_field) == 'bewerken'
    assert t9n_model.trans_field.msgid == 'edit'
    deactivate()


@pytest.mark.django_db
def test_trans_field_with_context():
    # we need to reload the instance from the db, so django converts the varchar value to a I18N object
    t9n_spring_device = _reload(TransTestModel.objects.create(trans_field=('mechanical device', 'spring')))
    t9n_spring_season = _reload(TransTestModel.objects.create(trans_field=('season', 'spring')))
    activate('en')
    assert str(t9n_spring_device.trans_field) == 'spring'
    assert str(t9n_spring_season.trans_field) == 'spring'
    deactivate()

    activate('nl')
    assert str(t9n_spring_device.trans_field) == 'veer'
    assert str(t9n_spring_season.trans_field) == 'lente'
    deactivate()


@pytest.mark.django_db
def test_trans_field_plural():
    # we need to reload the instance from the db, so django converts the varchar value to a I18N object
    t9n_spring_device = _reload(TransTestModel.objects.create(trans_field={'msgid': 'car', 'plural': 'cars'}))
    activate('en')
    assert t9n_spring_device.trans_field.trans(0) == 'cars'
    assert t9n_spring_device.trans_field.trans(1) == 'car'
    assert t9n_spring_device.trans_field.trans(2) == 'cars'

    activate('nl')
    assert t9n_spring_device.trans_field.trans(0) == "auto's"
    assert t9n_spring_device.trans_field.trans(1) == "auto"
    assert t9n_spring_device.trans_field.trans(2) == "auto's"

    activate('fr')
    assert t9n_spring_device.trans_field.trans(0) == "voiture"
    assert t9n_spring_device.trans_field.trans(1) == "voiture"
    assert t9n_spring_device.trans_field.trans(2) == "voitures"


@pytest.mark.django_db
def test_trans_field_plural_with_context():
    # we need to reload the instance from the db, so django converts the varchar value to a I18N object
    t9n_spring_device = _reload(TransTestModel.objects.create(trans_field={'i': 'tree', 'p': 'trees', 'c': 'in a forest'}))
    activate('en')
    assert t9n_spring_device.trans_field.trans(0) == 'trees'
    assert t9n_spring_device.trans_field.trans(1) == 'tree'
    assert t9n_spring_device.trans_field.trans(2) == 'trees'

    activate('nl')
    assert t9n_spring_device.trans_field.trans(0) == "bomen"
    assert t9n_spring_device.trans_field.trans(1) == "boom"
    assert t9n_spring_device.trans_field.trans(2) == "bomen"

    activate('fr')
    assert t9n_spring_device.trans_field.trans(0) == "arbre"
    assert t9n_spring_device.trans_field.trans(1) == "arbre"
    assert t9n_spring_device.trans_field.trans(2) == "arbres"


@pytest.mark.django_db
def test_trans_field_not_assigned():
    # we need to reload the instance from the db, so django converts the varchar value to a I18N object
    t9n_model = _reload(TransTestModel.objects.create())
    assert t9n_model.trans_field == ''


def test_t9n():
    assert len(T9N(msgid='test t9n')) == 8
    assert len(T9N(msgid='test t9n', msgctxt='my context')) == 8


def test_t9n_field():
    trans_field = TransField(max_length=128, null=False, blank=False)
    assert json.loads(trans_field.get_prep_value(T9N(msgid='test t9n', msgctxt='my context'))) == {
        'i': 'test t9n',
        'c': 'my context',
        'p': None
    }



# Copyright (c) 2017 by Cees van Wieringen <ceesvw@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
import json

from django import http


class JSONResponseMixin(object):
    def render_to_response(self, context, **http_response_kwargs):
        "Returns a JSON response containing 'context' as payload"
        return self.get_json_response(self.convert_context_to_json(context), **http_response_kwargs)

    def get_json_response(self, content, **http_response_kwargs):
        "Construct an `HttpResponse` object."
        return http.HttpResponse(content,
                                 content_type='application/json',
                                 **http_response_kwargs)

    def convert_context_to_json(self, context):
        "Convert the context dictionary into a JSON object"
        return json.dumps(context)




from setuptools import find_packages, setup

setup(
    name='django-l10n-extensions',
    version='1.0.6',
    author=u'Cees van Wieringen',
    author_email='ceesvw@gmail.com',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    include_package_data=True,
    install_requires=['GitPython==1.0.1', 'Django>=2', 'polib>=1.0'],
    url='https://github.com/ceasaro/django-l10n-extensions',
    license='',
    description=open('DESCRIPTION').read(),
    long_description=open('README.md').read(),
    zip_safe=False,
    key_words=['django', 'l10n', ]
)



from django.db import models

from django_l10n_extensions import measures


class L10n(models.Model):
    DISTANCE_CHOICES = [(distance_unit, measures.Distance.UNITS_REPR.get(distance_unit, distance_unit))
                        for distance_unit in measures.Distance.UNITS.keys()]

    AREA_CHOICES = [(area_unit, measures.Area.UNITS_REPR.get(area_unit, area_unit))
                    for area_unit in measures.Area.UNITS.keys()]

    WEIGHT_CHOICES = [(weight_unit, measures.Weight.UNITS_REPR.get(weight_unit, weight_unit))
                      for weight_unit in measures.Weight.UNITS.keys()]

    VOLUME_CHOICES = [(volume_unit, measures.Volume.UNITS_REPR.get(volume_unit, volume_unit))
                      for volume_unit in measures.Volume.UNITS.keys()]

    TEMP_CHOICES = [(temp_unit, measures.Temperature.UNITS_REPR.get(temp_unit, temp_unit))
                    for temp_unit in measures.Temperature.UNITS.keys()]

    VELOCITY_CHOICES = [(velocity_unit, measures.Velocity.UNITS_REPR.get(velocity_unit, velocity_unit))
                        for velocity_unit in measures.Velocity.UNITS.keys()]

    PRECIPITATION_CHOICES = [
        (precipitation_unit, measures.Precipitation.UNITS_REPR.get(precipitation_unit, precipitation_unit))
        for precipitation_unit in measures.Precipitation.UNITS.keys()]

    unit_distance = models.CharField(choices=DISTANCE_CHOICES, max_length=24, null=True, blank=True)
    unit_area = models.CharField(choices=AREA_CHOICES, max_length=24, null=True, blank=True)
    unit_weight= models.CharField(choices=WEIGHT_CHOICES, max_length=24, null=True, blank=True)
    unit_volume = models.CharField(choices=VOLUME_CHOICES, max_length=24, null=True, blank=True)
    unit_temp = models.CharField(choices=TEMP_CHOICES, max_length=24, null=True, blank=True)
    unit_velocity = models.CharField(choices=VELOCITY_CHOICES, max_length=24, null=True, blank=True)
    unit_precipitation = models.CharField(choices=PRECIPITATION_CHOICES, max_length=24, null=True,
                                          blank=True)

    def __unicode__(self):
        return u"L10n (Area= {area}) (Distance= {distance}) " \
               u"(Weight= {weight}) (Temperature= {temperature}) (Volume= {volume}) " \
               u"(Velocity = {velocity}) (Precipitation= {precipitation})".format(
            distance=self.get_unit_distance_display(),
            area=self.get_unit_area_display(),
            weight=self.get_unit_weight_display(),
            volume=self.get_unit_volume_display(),
            temperature=self.get_unit_temp_display(),
            velocity=self.get_unit_velocity_display(),
            precipitation=self.get_unit_precipitation_display(),
        )

    def as_dict(self):
        return {
            'unit_distance': self.unit_distance,
            'unit_distance_display': self.get_unit_distance_display(),

            'unit_area': self.unit_area,
            'unit_area_display': self.get_unit_area_display(),

            'unit_weight': self.unit_weight,
            'unit_weight_display': self.get_unit_weight_display(),

            'unit_volume': self.unit_volume,
            'unit_volume_display': self.get_unit_volume_display(),

            'unit_temp': self.unit_temp,
            'unit_temp_display': self.get_unit_temp_display(),

            'unit_windspeed': self.unit_velocity,
            'unit_windspeed_display': self.get_unit_velocity_display(),

            'unit_precipitation': self.unit_precipitation,
            'unit_precipitation_display': self.get_unit_precipitation_display(),
        }


DEFAULT_L10N = L10n(
    unit_distance=measures.Distance.METER,
    unit_area=measures.Area.SQUARE_METER,
    unit_weight=measures.Weight.GRAM,
    unit_volume=measures.Volume.LITER,
    unit_temp=measures.Temperature.CELSIUS,
    unit_velocity=measures.Velocity.MPS,
    unit_precipitation=measures.Precipitation.MM
)



# coding=utf-8
import pytest

from django_l10n_extensions.l10n_threading import activate as activate_l10n, deactivate as deactivate_l10n
from django_l10n_extensions.measures import Distance, Area

from tests.conftest import _reload
from tests.testapp.models import MeasuresTestModel


@pytest.mark.django_db
def test_area(l10n_nl):
    activate_l10n(l10n_nl)
    units = MeasuresTestModel.objects.create(area=240000, height=43, temp=18)
    units = _reload(units)
    assert units.area.sq_m == 240000
    assert units.area.ha == 24
    assert units.area.hectare == 24
    assert units.area.l10n == '240000.0 sq_m'
    assert units.area.as_l10n(decimal_pos=0) == '240000 sq_m'
    assert units.area.default_value == 240000
    assert units.height.default_value == 43


@pytest.mark.django_db
def test_distance(l10n_nl):
    activate_l10n(l10n_nl)
    units = MeasuresTestModel.objects.create(area=24, height=43, temp=18)
    units = _reload(units)
    assert units.height.m == 43
    assert units.height.meter == 43
    assert units.height.cm == 4300
    assert abs(units.height.ft - 141.076) < 0.001
    assert abs(units.height.foot - 141.076) < 0.001

    units.height = Distance(m=11)
    units.save()
    units = _reload(units)
    assert units.height.m == 11
    assert units.height.metre == 11


@pytest.mark.django_db
def test_temperature(l10n_nl, l10n_us):
    activate_l10n(l10n_nl)
    units = MeasuresTestModel.objects.create(area=Area(ha=24), height=43, temp=18)
    units = _reload(units)
    assert units.temp.C == 18.0
    assert units.temp.F == 64.4

    # change language, punctuation should alter
    units.temp = 0
    units.save()
    units = _reload(units)
    assert units.temp.Celsius == 0
    assert units.temp.F == 32

    activate_l10n(l10n_us)
    units.temp = 8  # 8 °F
    units.save()
    units = _reload(units)
    assert abs(units.temp.F - 8.00) < 0.01
    assert abs(units.temp.Celsius - -13.33) < 0.01
    deactivate_l10n()


@pytest.mark.django_db
def test_store():
    units = MeasuresTestModel.objects.create(area=Area(ha=1))
    units = _reload(units)
    assert units.area.ha == 1
    assert units.area.sq_km == 0.01

    units.area = 20000
    units.save()
    units = _reload(units)
    assert units.area.ha == 2
    assert units.area.sq_km == 0.02

    units.area = Area(ha=3)
    units.save()
    units = _reload(units)
    assert units.area.ha == 3
    assert units.area.sq_km == 0.03


@pytest.mark.django_db
def test_store_l10n(l10n_us):
    # set country to US, so values are stored in non default units.
    activate_l10n(l10n_us)
    # store values in the currently active unit.
    units = MeasuresTestModel.objects.create(area=1045462.7681142587, height=43, temp=0)
    units = _reload(units)
    assert abs(units.area.acre - 24) < 0.0001
    assert abs(units.area.ha - 9.7127) < 0.0001
    assert units.height.yard == 43
    assert abs(units.height.m - 39.3192) < 0.001
    assert abs(units.temp.F - 0.00) < 0.01
    assert abs(units.temp.C - -17.78) < 0.01
    # default values should return the value of the default unit.
    assert abs(units.area.default_value - 97126.66936) < 0.0001
    assert abs(units.height.default_value - 39.3192) < 0.01
    assert abs(units.temp.default_value - -17.78) < 0.01
    deactivate_l10n()


@pytest.mark.django_db
def test_velocity(l10n_nl, l10n_us):
    activate_l10n(l10n_nl)
    units = MeasuresTestModel.objects.create(windspeed=10)
    units = _reload(units)  # needed to convert int 10 to windspeed(10) in the units object
    assert units.windspeed.mps == 10
    assert units.windspeed.kmh == 36
    assert abs(units.windspeed.mph - 22.36936) < 0.00001

    assert l10n_nl.unit_velocity, 'mps'
    assert l10n_us.unit_velocity, 'mph'
    deactivate_l10n()


@pytest.mark.django_db
def test_precipitation(l10n_nl, l10n_eg, l10n_us):
    activate_l10n(l10n_nl)
    units = MeasuresTestModel.objects.create(precipitation=10)
    units = _reload(units)
    assert units.precipitation.mm == 10
    assert units.precipitation.inch == 10/25.4

    assert l10n_nl.unit_precipitation == 'mm'
    assert l10n_eg.unit_precipitation == 'mm'
    assert l10n_us.unit_precipitation == 'inch'
    deactivate_l10n()




# Copyright (c) 2010-2013 by Yaco Sistemas <ant30tx@gmail.com> or <goinnn@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.
from django.utils.encoding import force_str

html_escape_table = {
    "&": "&amp;",
    '"': "&quot;",
    "'": "&apos;",
    ">": "&gt;",
    "<": "&lt;",
}


def html_escape(text):
    if not text:
        return
    return "".join(html_escape_table.get(c, c) for c in force_str(text))




# First of all load and configure the django test app
import os

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.testapp.settings")
django.setup()

# now django is configured continue ...
import pytest
from django.utils.translation import pgettext, gettext, ngettext, npgettext

from django_l10n_extensions import measures
from django_l10n_extensions.models.models import L10n


def _reload(instance):
    """
    :param instance: a django model instance
    :return: a refreshed instance of the same django model instance
    """
    return type(instance).objects.get(pk=instance.pk)


def setup_translations():
    gettext('edit')
    pgettext('season', 'spring')
    pgettext('mechanical device', 'spring')
    npgettext('in a forest', 'tree', 'trees', 3)
    gettext('"double quote"')
    gettext("'single quote'")
    gettext("'single quote' with escaped \"double\"")
    gettext('"double quote" with exceped \'single\'')
    count = 3
    ngettext("You bought {} apple", "You bought {} apples", count)


@pytest.fixture
def l10n_nl():
    return L10n(
        unit_distance=measures.Distance.METER,
        unit_area=measures.Area.SQUARE_METER,
        unit_volume=measures.Volume.LITER,
        unit_weight=measures.Weight.GRAM,
        unit_temp=measures.Temperature.CELSIUS,
        unit_velocity=measures.Velocity.MPS,
        unit_precipitation=measures.Precipitation.MM,
    )


@pytest.fixture
def l10n_us():
    return L10n(
        unit_distance=measures.Distance.YARD,
        unit_area=measures.Area.SQUARE_FOOT,
        unit_volume=measures.Volume.GALLON,
        unit_weight=measures.Weight.POUND,
        unit_temp=measures.Temperature.FAHRENHEIT,
        unit_velocity=measures.Velocity.MPH,
        unit_precipitation=measures.Precipitation.INCH,
    )


@pytest.fixture
def l10n_eg():
    return L10n(
        unit_distance=measures.Distance.METER,
        unit_area=measures.Area.FEDDAN,
        unit_volume=measures.Volume.LITER,
        unit_weight=measures.Weight.GRAM,
        unit_temp=measures.Temperature.CELSIUS,
        unit_velocity=measures.Velocity.MPS,
        unit_precipitation=measures.Precipitation.MM,
    )



# Copyright (c) 2010-2013 by Yaco Sistemas <ant30tx@gmail.com> or <goinnn@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from django import template
from django.conf import settings
from django.urls import reverse

from django.utils.translation import get_language

from django_l10n_extensions import use_inline_trans


register = template.Library()


def get_static_url(subfix='inlinetrans'):
    static_prefix = getattr(settings, 'INLINETRANS_STATIC_URL', None)
    if static_prefix:
        return static_prefix
    static_prefix = getattr(settings, 'INLINETRANS_MEDIA_URL', None)
    if static_prefix:
        return static_prefix
    static_url = getattr(settings, 'STATIC_URL', getattr(settings, 'MEDIA_URL'))
    return '%s%s/' % (static_url, subfix)


def get_language_name(lang):
    for lang_code, lang_name in settings.LANGUAGES:
        if lang == lang_code:
            return lang_name


@register.inclusion_tag('inlinetrans/inline_header.html', takes_context=True)
def inlinetrans_static(context):
    tag_context = {
        'get_poentry_url': reverse('inlinetrans:get_po_entry'),
        'update_poentry_url': reverse('inlinetrans:update_poentry_url'),
        'can_translate': False,
        'INLINETRANS_STATIC_URL': get_static_url(),
        'INLINETRANS_MEDIA_URL': get_static_url(),  # backward compatible
        'request': context['request'],
    }
    user = context.get('user', None)
    if user and user.is_staff:
        tag_context.update({
            'can_translate': use_inline_trans,
            'is_staff': True,  # backward compatible
            'language': get_language_name(get_language()),
        })
    return tag_context




